@@ -1,4 +1,4 @@
-# $Id: POE.pm,v 1.169 2004/02/02 17:32:12 rcaputo Exp $
+# $Id: POE.pm,v 1.181 2004/11/25 19:44:12 rcaputo Exp $
 # Copyrights and documentation are after __END__.
 
 package POE;
@@ -7,15 +7,18 @@ use strict;
 use Carp;
 
 use vars qw($VERSION $REVISION);
-$VERSION = '0.2802';
-$REVISION = do {my@r=(q$Revision: 1.169 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = '0.30';
+$REVISION = do {my@r=(q$Revision: 1.181 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 sub import {
   my $self = shift;
 
+  my @loops    = grep(/^Loop\:\:/, @_);
   my @sessions = grep(/^(Session|NFA)$/, @_);
-  my @modules = grep(!/^(Kernel|Session|NFA)$/, @_);
+  my @modules  = grep(!/^(Kernel|Session|NFA|Loop)$/, @_);
 
+  croak "can't use multiple event loops at once"
+    if (@loops > 1);
   croak "POE::Session and POE::NFA export conflicting constants"
     if grep(/^(Session|NFA)$/, @sessions) > 1;
 
@@ -27,16 +30,32 @@ sub import {
     unshift @modules, 'Session';
   }
 
-  # Add Kernel back in, whether anybody wanted it or not.  Ensure that
-  # it comes before any sessions, since the sessions need to refer to
-  # constants defined in Kernel's namespace.
-  unshift @modules, 'Kernel';
-
-  my $package = (caller())[0];
-
+  my $package = caller();
   my @failed;
+
+  # Load POE::Kernel in the caller's package.  This is separate
+  # because we need to push POE::Loop classes through POE::Kernel's
+  # import().
+
+  {
+    my $loop = "";
+    if (@loops) {
+      $loop = "{ loop => '" . shift (@loops) . "' }";
+    }
+    my $code = "package $package; use POE::Kernel $loop;";
+    # warn $code;
+    eval $code;
+    if ($@) {
+      warn $@;
+      push @failed, "Kernel"
+    };
+  }
+
+  # Load all the others.
+
   foreach my $module (@modules) {
     my $code = "package $package; use POE::$module;";
+    # warn $code;
     eval($code);
     if ($@) {
       warn $@;
@@ -189,7 +208,7 @@ examples, documentation, and more.
 
 The developers of POE strive to make it as portable as possible.  If
 you discover a problem, please e-mail a report to
-<bug-POE@rt.cpan.org>.  If you can, include error messages, perl -V
+<bug-POE@rt.cpan.org>.  If you can, include error messages, C<perl -V>
 output, and/or test cases.  The more information you can provide, the
 quicker we can turn around a fix.  Patches are also welcome, of
 course.
@@ -411,7 +430,7 @@ the POE::Components for TCP clients and servers.
 
 =item Matt Cashner
 
-Matt Cashner is <eek@eekeek.org>, aka C<sungo>.  Matt is one of POE's
+Matt Cashner is <sungo@pobox.com>, aka C<sungo>.  Matt is one of POE's
 core developers.  He's spearheaded the movement to simplify POE for
 new users, flattening the learning curve and making the system more
 accessible to everyone.  He uses the system in mission critical
